function Vq=LPVLinearInterp(rho,V,rhoq,bs)
% Vq=LPVLinearInterp(rho,V,rhoq)
%
% This function performs 1-D linear interpolation for LPV systems.
% The syntax is similar to the function INTERP1 but is written 
% specifically for LPV systems with a single parameter.  The input V
% contains data defined on the parameter grid rho. The input V is either:
% i) a matrix of trim conditions, ii) an array of state-space models, 
% or iii) an array of matrices. The function computes Vq at the point
% rhoq by linearly interpolating the data in (rho,V).
%
% Inputs:
%  *rho = Ngrid-by-1 vector of parameter values. 
%  *V = Input data defined on the parameter grid as:
%       (i) Nx-by-Ngrid matrix of trim points where V(:,i) is the
%           trim condition at rho(i).
%       (ii) Ngrid-by-1 array of state-space models where V(:,:,i) is
%           the state-space model defined at rho(i).
%       (iii) N-by-N-by-Ngrid array of matrices where V(:,:,i) is the
%           matrix defined at rho(i).
%  *rhoq = Scalar value of parameter for interpolation. 
%
% Output:
%  *Vq = Interpolated value of V at rhoq. 
%

% Find location of rhoq in the vector rho
idx = find(rhoq>=rho,1,'last');
fac = (rhoq-rho(idx))/(rho(idx+1)-rho(idx));

% if isa(V,'double') && size(V,3)==1
%     % Matrix of Trim Conditions
%     Vq = (1-fac)*V(:,idx)+fac*V(:,idx+1);
% elseif isa(V,'double')
%     % Array of matrices
%     Vq = (1-fac)*V(:,:,idx)+fac*V(:,:,idx+1);
% else
%     % Array of systems
%     [A,B,C,D,Ts]=ssdata(V);
%     Aq = (1-fac)*A(:,:,idx)+fac*A(:,:,idx+1);
%     Bq = (1-fac)*B(:,:,idx)+fac*B(:,:,idx+1);
%     Cq = (1-fac)*C(:,:,idx)+fac*C(:,:,idx+1);
%     Dq = (1-fac)*D(:,:,idx)+fac*D(:,:,idx+1);
%     Vq = ss(Aq,Bq,Cq,Dq,Ts);
% end

if bs == 1
    % Matrix of Trim Conditions
    Vq = (1-fac)*V(idx,:)+fac*V(idx+1,:);    
elseif bs == 2
    % Array of systems
    [A,B,C,D,Ts]=ssdata(V);
    Aq = (1-fac)*A(:,:,idx)+fac*A(:,:,idx+1);
    Bq = (1-fac)*B(:,:,idx)+fac*B(:,:,idx+1);
    Cq = (1-fac)*C(:,:,idx)+fac*C(:,:,idx+1);
    Dq = (1-fac)*D(:,:,idx)+fac*D(:,:,idx+1);
    Vq = ss(Aq,Bq,Cq,Dq,Ts);
    
end
    
