function [bU,bV] = SteadyStateActDisk(nx,ny,hx,hy,Re,Cd,dt,Y,X,Nt,m,E,locTurb1,locTurb2,locTurbY,tol)

%-----------------------------------------------------------------------
% initial conditions

U = ones(nx,ny); 
V = zeros(nx,ny);

% initial perturbation to break symmetry
U = U + 0.1*exp(-1000*((X-0.25).^2+(Y-0.25).^2));

U = reshape(U,[],1);
V = reshape(V,[],1);

%-----------------------------------------------------------------------
% SFD

bU = U;
bV = V;
chi = 0.5;
Delt = 1;

%-----------------------------------------------------------------------
% setup 

% first derivatives
Dx = kron(speye(ny),D1z(nx,hx));
Dy = kron(D1p(ny,hy),speye(nx));

% Laplacian
L = kron(speye(ny),D2z(nx,hx)) + kron(D2p(ny,hy),spdiags([0;ones(nx-2,1);0],0,nx,nx));

% Cholesky requires positive definite so use -L
Lp = -L;
perp = symamd(Lp); 
Rp = chol(Lp(perp,perp)); 
Rpt = Rp';


%-----------------------------------------------------------------------
% time loop 

for step = 1:Nt

    % linear drag force (not realistic) 
    u_ib = E*U;
    v_ib = E*V;   
    fx = -u_ib;
    fy = -v_ib;
    
    % predict velocity (forward Euler)
    Ut = U + dt * (1/Re*(L*U) - U.*(Dx*U) - V.*(Dy*U) + Cd*E'*fx - chi*(U-bU));
    Vt = V + dt * (1/Re*(L*V) - U.*(Dx*V) - V.*(Dy*V) + Cd*E'*fy - chi*(V-bV));
    
    % boundary conditions (Dirichlet inflow, convective outflow)
    Ut = reshape(Ut,nx,ny);
    Vt = reshape(Vt,nx,ny);
    Ut(1,:) = 1;
    Vt(1,:) = 0;
    Ut(end,:) = U(end,:) - dt/hx*(U(end,:)-U(end-1,:));
    Vt(end,:) = V(end,:) - dt/hx*(V(end,:)-V(end-1,:));
    Ut = reshape(Ut,[],1);
    Vt = reshape(Vt,[],1);
   
    % Poisson eqn for pressure
    rhs = (Dx*Ut + Dy*Vt)/dt;        
    P(perp,1) = -Rp\(Rpt\rhs(perp));
    
    % correct velocity
    Unew = Ut - dt*Dx*P;
    V = Vt - dt*Dy*P;
    
    % base velocity
    bU = bU + dt * (U - bU) / Delt; 
    bV = bV + dt * (V - bV) / Delt; 
    
    resid(step) = norm(U-Unew);
    if resid(step) < tol
        return
    end
    U = Unew;
    
    % visualization
%     if (mod(step,m) == 0)
%         pcolor(X,Y,reshape(U,nx,ny)); shading interp; colorbar;
%         daspect([1 1 1]);
%         hold on;
%         plot([locTurb1 locTurb1],[locTurbY-0.5 locTurbY+0.5],'k','LineWidth',2);
%         plot([locTurb2 locTurb2],[locTurbY-0.5 locTurbY+0.5],'k','LineWidth',2);
%         hold off;
%         drawnow;
%         fprintf('%d %e\n', step, resid(step));
%     end;
    
end;
    
figure;
semilogy(resid);
    

end