%% LPV Model Order Reduction By Parameter-Varying Oblique Projection
%  paper submitted to the IEEE Transactions on Control Systems Technology
%  authors Julian Theis, Peter Seiler, Herbert Werner
%
% EXAMPLE 2: Wind Farm Aerodynamics
% Thanks to Jen Annoni for providing the actuator disk model

% last modified 05/02/2015 Julian Theis
format compact
close all
clear all
clc
addpath('./auxil')
%% Set up two actuator disks in domain

k = 20;                     % order of the reduced-order model
m = 50;                     % number of timesteps between snapshots
inp = 1;                    % number of inputs
outp = 1;                   % number of outputs

nx = 201; %was 201          % number of points in the x direction
ny = 51; %was 51            % number of points in the y direction
lx = 20;                    % length of box
ly = 5;                     % width of box
x = linspace(0,lx,nx);      % x dimension 
y = linspace(0,ly,ny);      % y dimension
[Y,X] = meshgrid(y,x);      % x,y grid

dx = lx/(nx-1);             % x spacing
dy = ly/(ny-1);             % y spacing

% Turbine locations
loc = 5;  %was 3            % number of diameters downstream from last turbine: wind speed measuremend point
locTurb1 = 5;               % location of turbine 1 (units: (Diameters), where D = 1 in this nondimensionalized code)
locTurb2 = locTurb1 + 5;    % location of turbine 2 (units: (Diameters), where D = 1 in this nondimensionalized code)
locTurbY = ly/2;
locX = locTurb1/dx - 1;     % x-grid point where turbine 1 is located
locX2 = locTurb2/dx - 1;    % x-grid point where turbine 2 is located
locY = locTurbY/dy - 1;     % y-grid point where turbines are located

U = 1;                      % nondimensionalized freestream

dt = 1e-2;                  % time step
Nt_ss = 5000;               % number of time steps for Steady State baseflow is recorded
Nt = 5000;                  % number of snapshots for forward and adjoint simulations and IODMD simulations
td = (loc/U);               % time delay between turbine and measurement location
tdm = (td/dt)/m;            % number of snapshots between turbines

%% Flow parameters
np = 5;                     % number of parameters to linearize about
Re = linspace(10,50,np);    % define Reynolds numbers for LPV system identification
Cd = 30;                    % coefficient of drag

%% Immersed Boundary Layer
%-----------------------------------------------------------------------
% setup immersed boundary for each turbine

E = sparse(30,nx*ny);
for i5 = 1:1
    for i4 = 1:2

        for i1 = 1:10
            for i2 = 1:4
                for i3 = 1:4
                    if i4 == 1
                        indx = locX +i3;
                    elseif i4 == 2
                        indx = locX2 +i3;
                    end                       
                    indy = (locY-5)+i1+i2 + 40*(i5-1);
                    cx = 0.25*(1+cos((-ly/2+i3)*pi/2));
                    cy = 0.25*(1+cos((-ly/2+i2)*pi/2));
                    E( ((i5-1)*3+i4)*10+i1,nx*(indy-1)+indx) = cx*cy;
                end;
            end;
        end;
        
    end;
end;

%% Steady State Act Disk Problem
tol = 10^(-5);                      % residual value (make sure the simulation has reached steady state in order to record baseflow (aka equilibrium point)
disp('Defining equilibrium points...')
for i = 1:np
    disp(sprintf('Re = %0.4f',Re(i)))
    [bU(i,:),bV(i,:)] = SteadyStateActDisk(nx,ny,dx,dy,Re(i),Cd,dt,Y,X,Nt_ss,m,E,locTurb1,locTurb2,locTurbY,tol);
end


%% Define Input to freestream 
B1 = InputActDisk(nx,ny);

%% Define Output - measurement point
C11 = zeros(1,nx*ny);
C12 = zeros(1,nx*ny);
C12 = reshape(C12,nx,ny);
C12(1+(locTurb2+loc)/dx,1+ly/2/dy) = 1;
C12 = reshape(C12,1,nx*ny);
C1 = sparse([C11 C12]);
        
%% Define Actuator location (asymmetric drag on wind turbines)
B2 = ActuatorActDisk(nx,ny,locTurb1,locY,dx,locX,locX2);

%% Define SISO system
B = B2(:,2);                    % input is only the last turbine
C = C1;                         % output is the measurement point downstream

%% BPOD GRAMIANS

%% Samples for controlability
Lc = zeros(size(B,1),Nt/m,np);
for ii = 1:np
Lc(:,:,ii) = ForwardActDisk(bU(ii,:)',bV(ii,:)',B,Re(ii),Cd,dt,nx,ny,dx,dy,X,Y,Nt,m,E,locTurb1,locTurb2,locTurbY);
end
%% Samples for observability
Lo = zeros(size(B,1),Nt/m,np);
for ii = 1:np
Lo(:,:,ii) = AdjointActDisk(bU(ii,:)',bV(ii,:)',C,Re(ii),Cd,dt,nx,ny,dx,dy,X,Y,Nt,m,E,locTurb1,locTurb2,locTurbY);
end

%% Approximate Balancing Subspace Calculation
nr = 6;
[Uh, LU, LUstack, Vi, Sh] = deal([]);
for ii=1:np   
    [Uh(:,:,ii), Sh(:,:,ii), ~]   = svd(Lc(:,:,ii)'*Lo(:,:,ii)); 
    figure(1000);semilogy(diag(Sh(:,:,ii)));hold all;
    
    LU(:,:,ii)           = orth(Lc(:,:,ii)*Uh(:,1:nr,ii));
    % local basis
    Vi(:,:,ii) = LU(:,:,ii);
    % stacked bases
    LUstack = [LUstack LU(:,:,ii)];
end

[Qbar, Sigbar, ~]   = svd(LUstack, 0);
figure;semilogy(diag(Sigbar),'ko')

Lo_p = pmat(Lo,rgrid(pgrid('Re',10:10:50)));
% parameter-independent basis space
V = Qbar(:,1:nr);
% parameter-varying test space
    Y = Lo_p'*V;
    W = Lo_p/Y';  % W = Lo*Y/(Y'*Y);
%
clearvars AData
for ii=1:np
    AData(:,:,ii) = ProjectActDisk(bU(ii,:)',bV(ii,:)',nx,ny,dx,dy,Re(ii),Cd,dt,W.Data(:,:,ii),V,nr,E,locTurb1);   
end

A_p  =  pmat(AData,(rgrid(pgrid('Re',10:10:50))));
sysr =  pss(A_p, W'*full(B), full(C)*V,1);


%% NONLINEAR SIMULATION 

% define simulation parameters 
NtLin = 5000;
Tmax = 50;
%
tTest = linspace(0,Tmax,NtLin);
NtTest = length(tTest);
yP = zeros(outp,NtTest);

% define signals
ReTest = mean(Re) - 10*sin(1*tTest);
% ReTest = mean(Re) - 20*sin(0.1*tTest);
uTest = 5*square(0.5*tTest);
dTest = 0*tTest;


% Nonlinear Simulation
%-----------------------------------------------------------------------
% initial conditions
bUInterp = LPVLinearInterp(Re,bU,ReTest(1),1)';
bVInterp = LPVLinearInterp(Re,bV,ReTest(1),1)'; 
U = bUInterp; 
V = bVInterp;

%-----------------------------------------------------------------------
% setup spatial operators

% first derivatives
Dx = kron(speye(ny),D1z(nx,dx));
Dy = kron(D1p(ny,dy),speye(nx));

% Laplacian
L = kron(speye(ny),D2z(nx,dx)) + kron(D2p(ny,dy),spdiags([0;ones(nx-2,1);0],0,nx,nx));

% Cholesky requires positive definite so use -L
Lp = -L;
perp = symamd(Lp); 
Rp = chol(Lp(perp,perp)); 
Rpt = Rp';


%-----------------------------------------------------------------------
% time loop 

uin = uTest(1);
distIn = dTest(1);
count = 0;
idx = 0;
Xsnapshot = 4000;

for step = 1:NtTest   
    
    if step == Xsnapshot
        Unonlinear = U;
        Vnonlinear = V;
    end
    
    ReIn = ReTest(step);
    uin = uTest(step);
    distIn = dTest(step);
    
    % update baseflow (trim solution)
    bUInterp = LPVLinearInterp(Re,bU,ReIn,1)';
    bVInterp = LPVLinearInterp(Re,bV,ReIn,1)';
    
    % linear drag force (not realistic) 
    u_ib = E*U;
    v_ib = E*V;   
    fx = -u_ib;
    fy = -v_ib;
    
    % predict velocity (forward Euler)
    Ut = U + dt * (1/ReIn*(L*U) - U.*(Dx*U) - V.*(Dy*U) + Cd*E'*fx + B(1:nx*ny,:)*uin + B1(1:nx*ny,:)*distIn);
    Vt = V + dt * (1/ReIn*(L*V) - U.*(Dx*V) - V.*(Dy*V) + Cd*E'*fy + B(nx*ny+1:2*nx*ny,:)*uin);
    
    % boundary conditions (Dirichlet inflow, convective outflow)
    Ut = reshape(Ut,nx,ny);
    Vt = reshape(Vt,nx,ny);
    Ut(1,:) = 1;
    Vt(1,:) = 0;
    Ut(end,:) = U(end,:) - dt/dx*(U(end,:)-U(end-1,:));
    Vt(end,:) = V(end,:) - dt/dx*(V(end,:)-V(end-1,:));
    Ut = reshape(Ut,[],1);
    Vt = reshape(Vt,[],1);
   
    % Poisson eqn for pressure
    rhs = (Dx*Ut + Dy*Vt)/dt;        
    P(perp,1) = -Rp\(Rpt\rhs(perp));
    
    % correct velocity
    Unew = Ut - dt*Dx*P;
    V = Vt - dt*Dy*P;

    yP(:,step) = C*([U; V] - [bUInterp; bVInterp]);

    U = Unew;
end;



figure(10)
    subplot(311)
    plot(tTest,uTest,'b','linewidth',3)
    ylabel('Turbine Forcing Input')
    subplot(312)
    plot(tTest,ReTest,'b','linewidth',3)
    ylabel('Freestream Velocity (parameter)')
    subplot(313)
    plot(tTest,yP,'b','linewidth',3)
    ylabel('Output')
    xlabel('Time (s)')




%% Validate LPV model 


%-----------------------------------------------------------------------
% initial conditions
bUInterp = LPVLinearInterp(Re,bU,ReTest(1),1)';
bVInterp = LPVLinearInterp(Re,bV,ReTest(1),1)';
    
U = bUInterp; 
V = bVInterp;

%-----------------------------------------------------------------------
% time loop 

[AA, BB, CC, DD] = ssdata(sysr);
baseflow = pmat(reshape([bU';bV'],2*size(bU',1),1,5),rgrid(pgrid('Re',Re)));


delz = zeros(nr,NtTest);
dely = zeros(1,NtTest);
zbar = zeros(nr,NtTest);
x0 = lpvinterp(baseflow,'Re',ReTest(1));
zbar(:,1) = lpvinterp(W,'Re',ReTest(1))'*x0;
ybar = zeros(1,NtTest);
ybar(:,1) = full(C)*x0;

for step = 1:NtTest-1   
        
    delu = uTest(step);
     
    Ak = double(lpvinterp(AA,'Re',ReTest(step)));
    Bk = double(lpvinterp(BB,'Re',ReTest(step)));
    Ck = double(lpvinterp(CC,'Re',ReTest(step)));
    Dk = 0;
    
    % Update State
    zdot = Ak*delz(:,step) + Bk*delu;
    delz(:,step+1) = delz(:,step)+zdot*dt ;
    
    % Update Output
    dely(:,step+1) = Ck*delz(:,step) + Dk*delu;
    
    if step == Xsnapshot
        xapprox = Qbar(:,1:nr)*delz(:,step-1);
        Ulpv = xapprox(1:nx*ny)+LPVLinearInterp(Re,bU,ReIn,1)';
        Vlpv = xapprox(1+nx*ny:end)+LPVLinearInterp(Re,bV,ReIn,1)';
    end
        
end;


figure(11)
    subplot(311)
    plot(tTest,uTest,'b','linewidth',3)
    ylabel('Turbine Forcing Input')
    subplot(312)
    plot(downsample(tTest,10),downsample(ReTest,10),'b','linewidth',3)
    ylabel('Freestream Velocity (parameter)')
    subplot(313)
    plot(tTest,yP,'b',tTest,dely,'r--','linewidth',3)
    ylabel('Output')
    legend('PDE','LPV')
    xlabel('Time (s)')
    
    
    
    
%% Flow Visualization
reso = -1:0.02:2;

figure(1001)
contour(reshape(bU(3,:),nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('baseflow U')

figure(1002)
contour(reshape(bV(3,:),nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('baseflow V')

figure(1003)
contour(reshape(Unonlinear',nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('flow U (nonlinear)')

figure(1004)
contour(reshape(Vnonlinear',nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('flow V (nonlinear)')

figure(1005)
contour(reshape(Ulpv',nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('flow U (LPV)')

figure(1006)
contour(reshape(Vlpv',nx,ny)',reso)
colorbar
caxis([-0.5 1.5])
    if exist('pmkmp','file') %use more perceptive colormap if available
        colormap(pmkmp(numel(reso),'CubicL'))
    end
title('flow V (LPV)')
